!
!     EAM FORCE & ENERGY CALCULATION
!
!     PRB 69, art.144113 (2004)  <== Parameter for Al doesn't work!!
!     X.W.Zhou, R.A.Johnson and H.N.G.Wadley 
!     Al parameter is taken from Acta Mater. 49, p.4005 (2001)
!
!     Fast calculation using mesh data
!     
!     CALCULATION OF FORCE CONSTANT (Nov. 08)
!     New bookkeep algorithm
!     

      subroutine eam_geam_AM_forceconst(hessian)
      use eam_geam_function
      use variables
      use constants
      use bookkeep_param
      use debug_variables
      implicit double precision (a-h,o-z)
      double precision :: rhob(n) ! rho-bar_i (density)
      double precision :: ki,kif
      double precision :: geampair,geamembed,geamden
      double precision :: geampaird,geamembedd,geamdend
      integer :: geamsp
      integer :: istat
      double precision :: hessian(n*3,n*3), inf_dis
      double precision :: rx_s(n),ry_s(n),rz_s(n)

      istat=1

      rc=maxval(gre)*2.0d0*ang
      if (frc<rc) then
         frc=rc*1.15d0
         print *,'### WARNING: frc is corrected (GEAM)'
         iwarn=iwarn+1
      endif
      rc2=rc**2

      rhob=0.0d0


!   First Loop: Chg dns calc
!      kns=0
!      do 4200 k=1,n
!         kn=knum(k)
!         if (kn.eq.0) go to 4200
!         do 4300 jk=1,kn
!            kns=kns+1
!            j=idpt(kns,1)
!            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
!            xx=rx(k)-rx(j)
!            yy=ry(k)-ry(j)
!            zz=rz(k)-rz(j)
!            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
!            if (istat == 1) then
!               r2=xx**2+yy**2+zz**2
!               if (r2.lt.rc2) then
!                  r=sqrt(r2)
!                  rhob(k)=rhob(k)+geamden(r/ang,geamsp(asp(j)))
!                  if (j/=k) then
!                     rhob(j)=rhob(j)+geamden(r/ang,geamsp(asp(k)))
!                  endif
!               endif
!            endif
! 4300    continue
! 4200 continue

!     Calculation of potential energy
      do i=1,n
         enpoteach(i)=geamembed(rhob(i),geamsp(asp(i)))*ev
      enddo


!   Second Loop: Force constant calc
      inf_dis=0.0005*ang
      inf_dis=0.05*ang
!      rcc=(rc+inf_dis)*2.5; rcc2=rcc**2 !!! range of influence
      rcc=(rc+inf_dis)*2.0; rcc2=rcc**2 !!! range of influence
      hessian=0.0d0
      rx_s(1:n)=rx(1:n); ry_s(1:n)=ry(1:n); rz_s(1:n)=rz(1:n)
      outloop: do ifc=1,6
      inloop: do l=1,n
         rx(1:n)=rx_s(1:n); ry(1:n)=ry_s(1:n); rz(1:n)=rz_s(1:n)
         fx(1:n)=0.0d0; fy(1:n)=0.0d0; fz(1:n)=0.0d0
         if (ifc==1) then; rx(l)=rx(l)+inf_dis; endif
         if (ifc==2) then; rx(l)=rx(l)-inf_dis; endif
         if (ifc==3) then; ry(l)=ry(l)+inf_dis; endif
         if (ifc==4) then; ry(l)=ry(l)-inf_dis; endif
         if (ifc==5) then; rz(l)=rz(l)+inf_dis; endif
         if (ifc==6) then; rz(l)=rz(l)-inf_dis; endif

!   First Loop: Chg dns calc
      rhob=0.0d0
      kns=0
      do 4200 k=1,n
         kn=knum(k)
         if (kn.eq.0) go to 4200
         do 4300 jk=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(k)-rx(j)
            yy=ry(k)-ry(j)
            zz=rz(k)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            if (istat == 1) then
               r2=xx**2+yy**2+zz**2
               if (r2.lt.rc2) then
                  r=sqrt(r2)
                  rhob(k)=rhob(k)+geamden(r/ang,geamsp(asp(j)))
                  if (j/=k) then
                     rhob(j)=rhob(j)+geamden(r/ang,geamsp(asp(k)))
                  endif
               endif
            endif
 4300    continue
 4200 continue

      kns=0
      iloop: do i=1,n
         kn=knum(i)
         if (kn.eq.0) cycle
         disx=abs(rx_s(i)-rx_s(l)); disy=abs(ry_s(i)-ry_s(l)); disz=abs(rz_s(i)-rz_s(l))
!         disx=abs(rx(i)-rx(l)); disy=abs(ry(i)-ry(l)); disz=abs(rz(i)-rz(l))
!         if (pbcx.and.(disx>rc*2)) disx=disx-hmat(1,1)
!         if (pbcy.and.(disy>rc*3)) disy=disy-hmat(2,2)
!         if (pbcz.and.(disz>rc*3)) disz=disz-hmat(3,3)
         if (pbcx) disx=disx-int(disx/hmat(1,1))*hmat(1,1)
         if (pbcy) disy=disy-int(disy/hmat(2,2))*hmat(2,2)
         if (pbcz) disz=disz-int(disz/hmat(3,3))*hmat(3,3)
         if (pbcx.and.(disx>hmat(1,1)/2.0)) disx=disx-hmat(1,1)
         if (pbcy.and.(disy>hmat(2,2)/2.0)) disy=disy-hmat(2,2)
         if (pbcz.and.(disz>hmat(3,3)/2.0)) disz=disz-hmat(3,3)
         dist2=disx**2+disy**2+disz**2

         do 4600 jk=1,kn
            kns=kns+1
            if (dist2>rc2*9) go to 4601
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j)
            yy=ry(i)-ry(j)
            zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            if (istat == 1) then
               r2=xx**2+yy**2+zz**2
               if (r2.lt.rc2) then
                  r=sqrt(r2)
                  vij=geampair(r/ang,geamsp(asp(i)),geamsp(asp(j)))*ev
                  dv=geampaird(r/ang,geamsp(asp(i)),geamsp(asp(j)))
                  enpoteach(i)=enpoteach(i)+vij/2.0d0
                  vijsum=vijsum+vij/2.0d0
                  if (j/=i) then
                     enpoteach(j)=enpoteach(j)+vij/2.0d0
                     vijsum=vijsum+vij/2.0d0
                  endif

                  df_i=geamembedd(rhob(i),geamsp(asp(i)))
                  df_j=geamembedd(rhob(j),geamsp(asp(j)))

                  dr_i=geamdend(r/ang,geamsp(asp(i)))
                  dr_j=geamdend(r/ang,geamsp(asp(j)))

                  f=-(df_i*dr_j+df_j*dr_i+dv)*ev/ang

                  ki=f/r
                  kif=ki/2.0d0
                  if (j==i) ki=ki/2.0d0
                  fx(i)=fx(i)+ki*xx
                  fy(i)=fy(i)+ki*yy
                  fz(i)=fz(i)+ki*zz
                  fx(j)=fx(j)-ki*xx
                  fy(j)=fy(j)-ki*yy
                  fz(j)=fz(j)-ki*zz
               endif
            endif
4601        continue
4600        continue
      end do iloop


      i=l  !!! self term
         if (ifc==1) then
            hessian((l-1)*3+1,(i-1)*3+1)=hessian((l-1)*3+1,(i-1)*3+1)+fx(i)
            hessian((l-1)*3+1,(i-1)*3+2)=hessian((l-1)*3+1,(i-1)*3+2)+fy(i)
            hessian((l-1)*3+1,(i-1)*3+3)=hessian((l-1)*3+1,(i-1)*3+3)+fz(i)
         else if (ifc==2) then
            hessian((l-1)*3+1,(i-1)*3+1)=hessian((l-1)*3+1,(i-1)*3+1)-fx(i)
            hessian((l-1)*3+1,(i-1)*3+2)=hessian((l-1)*3+1,(i-1)*3+2)-fy(i)
            hessian((l-1)*3+1,(i-1)*3+3)=hessian((l-1)*3+1,(i-1)*3+3)-fz(i)
         else if (ifc==3) then
            hessian((l-1)*3+2,(i-1)*3+1)=hessian((l-1)*3+2,(i-1)*3+1)+fx(i)
            hessian((l-1)*3+2,(i-1)*3+2)=hessian((l-1)*3+2,(i-1)*3+2)+fy(i)
            hessian((l-1)*3+2,(i-1)*3+3)=hessian((l-1)*3+2,(i-1)*3+3)+fz(i)
         else if (ifc==4) then
            hessian((l-1)*3+2,(i-1)*3+1)=hessian((l-1)*3+2,(i-1)*3+1)-fx(i)
            hessian((l-1)*3+2,(i-1)*3+2)=hessian((l-1)*3+2,(i-1)*3+2)-fy(i)
            hessian((l-1)*3+2,(i-1)*3+3)=hessian((l-1)*3+2,(i-1)*3+3)-fz(i)
         else if (ifc==5) then
            hessian((l-1)*3+3,(i-1)*3+1)=hessian((l-1)*3+3,(i-1)*3+1)+fx(i)
            hessian((l-1)*3+3,(i-1)*3+2)=hessian((l-1)*3+3,(i-1)*3+2)+fy(i)
            hessian((l-1)*3+3,(i-1)*3+3)=hessian((l-1)*3+3,(i-1)*3+3)+fz(i)
         else if (ifc==6) then
            hessian((l-1)*3+3,(i-1)*3+1)=hessian((l-1)*3+3,(i-1)*3+1)-fx(i)
            hessian((l-1)*3+3,(i-1)*3+2)=hessian((l-1)*3+3,(i-1)*3+2)-fy(i)
            hessian((l-1)*3+3,(i-1)*3+3)=hessian((l-1)*3+3,(i-1)*3+3)-fz(i)
         endif

      kns=0  !!! correlation term
      if (l>1) then
         do i=1,l-1
            kns=kns+knum(i)
         enddo
      endif
      ipre=0

      do i=1,n
         if (i==l) cycle  !!! avoid self interaction
         if (i==ipre) cycle  !!! avoid double counting
         ipre=i
         xx=rx_s(l)-rx_s(i); yy=ry_s(l)-ry_s(i); zz=rz_s(l)-rz_s(i)
!!! search replica (using calc_replica leads to error)
         if (xx> rcc) xx=xx-hmat(1,1)
         if (xx<-rcc) xx=xx+hmat(1,1)
         if (yy> rcc) yy=yy-hmat(2,2)
         if (yy<-rcc) yy=yy+hmat(2,2)
         if (zz> rcc) zz=zz-hmat(3,3)
         if (zz<-rcc) zz=zz+hmat(3,3)
         dist2=xx**2+yy**2+zz**2
         if (dist2>rcc2) cycle  !!!! skip unless i-l distance < rcc

         if (ifc==1) then
            hessian((l-1)*3+1,(i-1)*3+1)=hessian((l-1)*3+1,(i-1)*3+1)+fx(i)
            hessian((l-1)*3+1,(i-1)*3+2)=hessian((l-1)*3+1,(i-1)*3+2)+fy(i)
            hessian((l-1)*3+1,(i-1)*3+3)=hessian((l-1)*3+1,(i-1)*3+3)+fz(i)
         else if (ifc==2) then
            hessian((l-1)*3+1,(i-1)*3+1)=hessian((l-1)*3+1,(i-1)*3+1)-fx(i)
            hessian((l-1)*3+1,(i-1)*3+2)=hessian((l-1)*3+1,(i-1)*3+2)-fy(i)
            hessian((l-1)*3+1,(i-1)*3+3)=hessian((l-1)*3+1,(i-1)*3+3)-fz(i)
         else if (ifc==3) then
            hessian((l-1)*3+2,(i-1)*3+1)=hessian((l-1)*3+2,(i-1)*3+1)+fx(i)
            hessian((l-1)*3+2,(i-1)*3+2)=hessian((l-1)*3+2,(i-1)*3+2)+fy(i)
            hessian((l-1)*3+2,(i-1)*3+3)=hessian((l-1)*3+2,(i-1)*3+3)+fz(i)
         else if (ifc==4) then
            hessian((l-1)*3+2,(i-1)*3+1)=hessian((l-1)*3+2,(i-1)*3+1)-fx(i)
            hessian((l-1)*3+2,(i-1)*3+2)=hessian((l-1)*3+2,(i-1)*3+2)-fy(i)
            hessian((l-1)*3+2,(i-1)*3+3)=hessian((l-1)*3+2,(i-1)*3+3)-fz(i)
         else if (ifc==5) then
            hessian((l-1)*3+3,(i-1)*3+1)=hessian((l-1)*3+3,(i-1)*3+1)+fx(i)
            hessian((l-1)*3+3,(i-1)*3+2)=hessian((l-1)*3+3,(i-1)*3+2)+fy(i)
            hessian((l-1)*3+3,(i-1)*3+3)=hessian((l-1)*3+3,(i-1)*3+3)+fz(i)
         else if (ifc==6) then
            hessian((l-1)*3+3,(i-1)*3+1)=hessian((l-1)*3+3,(i-1)*3+1)-fx(i)
            hessian((l-1)*3+3,(i-1)*3+2)=hessian((l-1)*3+3,(i-1)*3+2)-fy(i)
            hessian((l-1)*3+3,(i-1)*3+3)=hessian((l-1)*3+3,(i-1)*3+3)-fz(i)
         endif
      enddo

      end do inloop
      end do outloop

      hessian=-hessian/inf_dis/2.0
      rx(1:n)=rx_s(1:n); ry(1:n)=ry_s(1:n); rz(1:n)=rz_s(1:n)

      end



