!
! Search for atom in replica cell which exists within cutoff radius
! Input
!  sdx,sdy,sdz : components of r-r0 vector
!  rad         : cutoff radius (e.g. rc for force calc, frc for bookkeep)
!  ix,iy,iz    : indicate which cell is searched
!                e.g. primitive cell is searched if =0
!                     adjoining cell is searched if =1
! Output
!  sdx,sdy,sdz : components of r-r0 within cutoff sphere
!  istat       : search result status (0: no atom, 1: found)
!
      subroutine search_neighbor(sdx,sdy,sdz,rad,ix,iy,iz,istat)
      use constants
      use variables, only : hmat,pbcx,pbcy,pbcz
      use debug_variables
      double precision :: sdx,sdy,sdz,rad
      double precision :: sdxi,sdyi,sdzi
      integer :: istat,ix,iy,iz
      istat=1
      sdxi=sdx
      sdyi=sdy
      sdzi=sdz
      if (abs(sdx).le.rad) then
         ix=0
         go to 20
      endif
      if (pbcx) then
         if (sdx.lt.0) then
            sdxi=sdx+hmat(1,1)
            sdyi=sdy+hmat(2,1)
            sdzi=sdz+hmat(3,1)
            if (abs(sdxi).le.rad) then
               ix=1
               go to 20
            endif
         else 
            sdxi=sdx-hmat(1,1)
            sdyi=sdy-hmat(2,1)
            sdzi=sdz-hmat(3,1)
            if (abs(sdxi).le.rad) then
               ix=-1
               go to 20
            endif
         endif
      endif
      go to 99
 20   sdx=sdxi
      sdy=sdyi
      sdz=sdzi
      if (abs(sdy).le.rad) then
         iy=0
         go to 40
      endif
      if (pbcy) then
         if (sdy.lt.0) then
            sdxi=sdx+hmat(1,2)
            sdyi=sdy+hmat(2,2)
            sdzi=sdz+hmat(3,2)
            if (abs(sdyi).le.rad) then
               iy=1
               go to 40
            endif
         else
            sdxi=sdx-hmat(1,2)
            sdyi=sdy-hmat(2,2)
            sdzi=sdz-hmat(3,2)
            if (abs(sdyi).le.rad) then
               iy=-1
               go to 40
            endif
         endif
      endif
      go to 99
 40   sdx=sdxi
      sdy=sdyi
      sdz=sdzi
      if (abs(sdz).le.rad) then
         iz=0
         go to 60
      endif
      if (pbcz) then
         if (sdz.lt.0) then
            sdxi=sdx+hmat(1,3)
            sdyi=sdy+hmat(2,3)
            sdzi=sdz+hmat(3,3)
            if (abs(sdzi).le.rad) then
               iz=1
               go to 60
            endif
         else
            sdxi=sdx-hmat(1,3)
            sdyi=sdy-hmat(2,3)
            sdzi=sdz-hmat(3,3)
            if (abs(sdzi).le.rad) then
               iz=-1
               go to 60
            endif
         endif
      endif
      go to 99
 60   sdx=sdxi
      sdy=sdyi
      sdz=sdzi
      istat = 1
! Check if self-interaction was counted
      if ((abs(sdx)<1.0d-15).and.(abs(sdy)<1.0d-15).and.(abs(sdz)<1.0d-15)) istat=0
      return
 99   istat = 0
      end
