!
!   Stillinger-Weber potential
!   for Silicon
!    Frank H. Stillinger and Thomas A. Weber
!    "Computer simulation of local order in condensed phases of silicon"
!    Physical Review B, Vol. 31, No. 8, pp.5262-5271 (1985)
!
!   calculation of force, total energy, stress
!
!   First written on May 8 (2006)
!   Completed on June 1 (2006)

      module sw_function
      double precision :: swaa, swbb, swa, swlam, swgam
      integer :: swp, swq
      end module

      subroutine sw
      use sw_function
      use variables
      use constants
      use bookkeep_param
      use debug_variables
      implicit double precision (a-h,o-z)
      double precision :: ki
      integer :: istat
      double precision :: x2(n),y2(n),z2(n),a1(3),a2(3),a3(3),para(7)
      if (bookkeep_algo /= 2) then
         stop 'Error in tersoff.F: Bookkeep algo is not DOUBLE'
      endif

!--Stillinger-Weber
      pi=4.0d0*atan(1.0d0)
      eps=1.0d-30
      ev2j=1.6021892d-19        !eV -> J
      j2ev=1.0d0/ev2j           !J -> eV
!  PARAMETER 
!  PRB 31 p5262 (1985)
      swaa=7.049556277d0
      swbb=0.6022245584d0
      swp=4
      swq=0
      swa=1.80d0
      swlam=21.0d0
      swgam=1.20d0
      swsig=0.20951d-9
      sweps=3.4723d-19
!--

      istat=1

      rc=swa*swsig
      rc2=rc**2


!   Virial term reset
      virx=0.0d0; viry=0.0d0; virz=0.0d0
      dmat=0.0d0
      satom=0.0d0               ! Atomic stress

!   Loop
      enpot=0.0d0
      enpoteach=0.0d0
      fx(1:n)=0.0d0
      fy(1:n)=0.0d0
      fz(1:n)=0.0d0
      kns=0
      iloop: do i=1,n
         rix=rx(i)
         riy=ry(i)
         riz=rz(i)
         kn=knum(i)
         if (kn.eq.0) cycle
!         do 200 jj=1,nc
         kns0=kns
         do 200 jcount=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j); yy=ry(i)-ry(j); zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            rjx=rx(i)-xx; rjy=ry(i)-yy; rjz=rz(i)-zz
            rij2=(rjx-rix)**2+(rjy-riy)**2+(rjz-riz)**2
            if (rij2.le.eps) go to 201
            if (rij2.ge.rc2) go to 201
               rij=sqrt(rij2)
               rijs=rij/swsig
               u2ij=sweps*swaa*(swbb*rijs**(-swp)-rijs**(-swq)) &
               *dexp((rijs-swa)**(-1))
!     urepij=fij*teraa*exp(-terlambda*rij)
               ad1=sweps/swsig*dexp((rijs-swa)**(-1)) &
               *swaa*(-dble(swp)*swbb*rijs**(-swp-1)+dble(swq)*rijs**(-swq-1) &
               +(swbb*rijs**(-swp)-rijs**(-swq))*(-(rijs-swa)**(-2)))
               fx(i)=fx(i)+ad1*(rjx-rix)/rij
               fy(i)=fy(i)+ad1*(rjy-riy)/rij
               fz(i)=fz(i)+ad1*(rjz-riz)/rij
!!     Atomic stress
               ad1=0.5d0*ad1/rij
               adx=rjx-rix; ady=rjy-riy; adz=rjz-riz
               satom(i,1,1)=satom(i,1,1)+ad1*adx*adx
               satom(i,1,2)=satom(i,1,2)+ad1*ady*adx
               satom(i,2,2)=satom(i,2,2)+ad1*ady*ady
               satom(i,1,3)=satom(i,1,3)+ad1*adz*adx
               satom(i,2,3)=satom(i,2,3)+ad1*adz*ady
               satom(i,3,3)=satom(i,3,3)+ad1*adz*adz
!--------------calc 3-body term---------------------
               u3ij=0.0d0
               knsk=kns0
               do 300 kcount=1,kn
                  knsk=knsk+1
                  k=idpt(knsk,1)
                  ix=idpt(knsk,2); iy=idpt(knsk,3); iz=idpt(knsk,4)
                  xx=rx(i)-rx(k); yy=ry(i)-ry(k); zz=rz(i)-rz(k)
                  call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
                  rkx=rx(i)-xx; rky=ry(i)-yy; rkz=rz(i)-zz
                  rik2=(rkx-rix)**2+(rky-riy)**2+(rkz-riz)**2

                  if (rik2.le.eps) go to 301
                  if (rik2.ge.rc2) go to 301
                  rik=sqrt(rik2)
                  riks=rik/swsig
                  rjk2=(rkx-rjx)**2+(rky-rjy)**2+(rkz-rjz)**2
                  if (rjk2.le.eps) go to 301
                  rjk=sqrt(rjk2)
                  rjks=rjk/swsig
                  cosjik=(rik2+rij2-rjk2)/(2.0d0*rik*rij)
                  cosijk=(rij2+rjk2-rik2)/(2.0d0*rjk*rij)
                  cosjki=(rjk2+rik2-rij2)/(2.0d0*rjk*rik)

                  u3ij=u3ij+sweps*swh(rijs,riks,cosjik)

                  dhdr1=-swgam/swsig*(rijs-swa)**(-2)*swh(rijs,riks,cosjik)
                  dhdr2=-swgam/swsig*(riks-swa)**(-2)*swh(rijs,riks,cosjik)

                  dhdc1=dswh(rijs,riks,cosjik)
                  dhdc2=dswh(rijs,rjks,cosijk)
                  dhdc3=dswh(riks,rjks,cosjki)

                  fx(i)=fx(i)+0.5d0*sweps*dhdr1*(rjx-rix)/rij
                  fy(i)=fy(i)+0.5d0*sweps*dhdr1*(rjy-riy)/rij
                  fz(i)=fz(i)+0.5d0*sweps*dhdr1*(rjz-riz)/rij
                  fx(i)=fx(i)+0.5d0*sweps*dhdr2*(rkx-rix)/rik
                  fy(i)=fy(i)+0.5d0*sweps*dhdr2*(rky-riy)/rik
                  fz(i)=fz(i)+0.5d0*sweps*dhdr2*(rkz-riz)/rik
                  fx(i)=fx(i)+0.5d0*sweps*dhdc1/rik2/rij2 &
                  *(cosijk*rik*rjk*(rjx-rix)+cosjki*rij*rjk*(rkx-rix))
                  fy(i)=fy(i)+0.5d0*sweps*dhdc1/rik2/rij2 &
                  *(cosijk*rik*rjk*(rjy-riy)+cosjki*rij*rjk*(rky-riy))
                  fz(i)=fz(i)+0.5d0*sweps*dhdc1/rik2/rij2 &
                  *(cosijk*rik*rjk*(rjz-riz)+cosjki*rij*rjk*(rkz-riz))

                  fx(j)=fx(j)-0.5d0*sweps*dhdr1*(rjx-rix)/rij
                  fy(j)=fy(j)-0.5d0*sweps*dhdr1*(rjy-riy)/rij
                  fz(j)=fz(j)-0.5d0*sweps*dhdr1*(rjz-riz)/rij
                  fx(k)=fx(k)-0.5d0*sweps*dhdr2*(rkx-rix)/rik
                  fy(k)=fy(k)-0.5d0*sweps*dhdr2*(rky-riy)/rik
                  fz(k)=fz(k)-0.5d0*sweps*dhdr2*(rkz-riz)/rik
                  fx(j)=fx(j)-0.5d0*sweps*dhdc1* &
                  (1.0d0/rij/rik*(rkx-rjx)+rjk*cosijk/rij2/rik*(rjx-rix))
                  fy(j)=fy(j)-0.5d0*sweps*dhdc1* &
                  (1.0d0/rij/rik*(rky-rjy)+rjk*cosijk/rij2/rik*(rjy-riy))
                  fz(j)=fz(j)-0.5d0*sweps*dhdc1* &
                  (1.0d0/rij/rik*(rkz-rjz)+rjk*cosijk/rij2/rik*(rjz-riz))
                  fx(k)=fx(k)-0.5d0*sweps*dhdc1* &
                  (1.0d0/rik/rij*(rjx-rkx)+rjk*cosjki/rik2/rij*(rkx-rix))
                  fy(k)=fy(k)-0.5d0*sweps*dhdc1* &
                  (1.0d0/rik/rij*(rjy-rky)+rjk*cosjki/rik2/rij*(rky-riy))
                  fz(k)=fz(k)-0.5d0*sweps*dhdc1* &
                  (1.0d0/rik/rij*(rjz-rkz)+rjk*cosjki/rik2/rij*(rkz-riz))

!!     Atomic stress
                  ad1=0.5d0*sweps*dhdr1/rij
                  ad2=0.5d0*sweps*dhdr2/rik
                  dxij=rjx-rix;dyij=rjy-riy;dzij=rjz-riz
                  dxik=rkx-rix;dyik=rky-riy;dzik=rkz-riz
                  dxjk=rkx-rjx;dyjk=rky-rjy;dzjk=rkz-rjz
                  satom(i,1,1)=satom(i,1,1)+ad1*dxij*dxij+ad2*dxik*dxik
                  satom(i,1,2)=satom(i,1,2)+ad1*dyij*dxij+ad2*dyik*dxik
                  satom(i,2,2)=satom(i,2,2)+ad1*dyij*dyij+ad2*dyik*dyik
                  satom(i,1,3)=satom(i,1,3)+ad1*dzij*dxij+ad2*dzik*dxik
                  satom(i,2,3)=satom(i,2,3)+ad1*dzij*dyij+ad2*dzik*dyik
                  satom(i,3,3)=satom(i,3,3)+ad1*dzij*dzij+ad2*dzik*dzik

                  ad0=0.5d0*sweps*dhdc1
                  ad1=-1.0d0/rik/rij
                  ad2=1.0d0/rik/rij-cosjik/rik**2
                  ad3=1.0d0/rik/rij-cosjik/rij**2
                  satom(i,1,1)=satom(i,1,1)+ad0*(ad1*dxjk*dxjk+ad2*dxik*dxik+ad3*dxij*dxij)
                  satom(i,1,2)=satom(i,1,2)+ad0*(ad1*dyjk*dxjk+ad2*dyik*dxik+ad3*dyij*dxij)
                  satom(i,2,2)=satom(i,2,2)+ad0*(ad1*dyjk*dyjk+ad2*dyik*dyik+ad3*dyij*dyij)
                  satom(i,1,3)=satom(i,1,3)+ad0*(ad1*dzjk*dxjk+ad2*dzik*dxik+ad3*dzij*dxij)
                  satom(i,2,3)=satom(i,2,3)+ad0*(ad1*dzjk*dyjk+ad2*dzik*dyik+ad3*dzij*dyij)
                  satom(i,3,3)=satom(i,3,3)+ad0*(ad1*dzjk*dzjk+ad2*dzik*dzik+ad3*dzij*dzij)

 301              continue
 300           continue
               uij=u2ij+u3ij
!               enpot=enpot+uij*0.5d0
               enpoteach(i)=enpoteach(i)+uij*0.5d0
 201           continue
 200     continue
      end do iloop

!---Etot---
      enpot=0.0d0
      do i=1,n
         enpot=enpot+enpoteach(i)
      enddo

!!     Atomic stress
      do i=1,n
         satom(i,2,1)=satom(i,1,2)
         satom(i,3,1)=satom(i,1,3)
         satom(i,3,2)=satom(i,2,3)
      enddo
      dmat=0.0d0
! The sign of dmat is opposite to stress tensor's (dmat is the "virial" tensor)
      do i=1,n
         dmat(1:3,1:3)=dmat(1:3,1:3)-satom(i,1:3,1:3)
      enddo
      virx=dmat(1,1); viry=dmat(2,2); virz=dmat(3,3)

      satom=satom*dble(n)/v

      end

      double precision function swh(r1,r2,cost)
      use sw_function
      implicit double precision (a-h,o-z)
      swh=swlam*dexp(swgam*(r1-swa)**(-1)+swgam*(r2-swa)**(-1))*(cost+1.0d0/3.0d0)**2
      end
      double precision function dswh(r1,r2,cost)
      use sw_function
      implicit double precision (a-h,o-z)
      dswh=swlam*dexp(swgam*(r1-swa)**(-1)+swgam*(r2-swa)**(-1))*2.0d0*(cost+1.0d0/3.0d0)
      end

!------------------ The following subroutine (written by A.Kushima) is not used ---------------------
      subroutine calculate3(nacount,i,j,k,xij,yij,zij,rij, &
      rrij,x2,y2,z2,a1,a2,a3 &
      ,fx,fy,fz,para,epot,xik,yik,zik,xjk,yjk,zjk)
      IMPLICIT DOUBLE PRECISION(A,B,D-H,O-Z),INTEGER(I-N),COMPLEX*16(C)
      dimension x2(nacount),y2(nacount),z2(nacount),a1(3),a2(3),a3(3)
      dimension fx(nacount),fy(nacount),fz(nacount)
      dimension para(7)

      para(1)=7.049556277d0     !swaa
      para(2)=0.6022245584D0    !swbb
      para(3)=1.8d0             !swa
      para(4)=21.0d0            !swlam
      para(5)=1.2d0             !swsig
      para(6)=0.20951d0         !sweps
      para(7)=3.4723d-19        

      rc=para(3)*para(6)        
      rc2=rc**2
      epot=0.0d0
      
!      xk=x2(k)+a1(1)*kx+a2(1)*ky+a3(1)*kz
!      yk=y2(k)+a1(2)*kx+a2(2)*ky+a3(2)*kz
!      zk=z2(k)+a1(3)*kx+a2(3)*ky+a3(3)*kz
!      xik=xk-x2(i)
!      yik=yk-y2(i)
!      zik=zk-z2(i)
!      xjk=xk-xj
!      yjk=yk-yj
!      zjk=zk-zj
!      print *, rc,xjk,yjk,zjk
      if(abs(xik).gt.rc) goto 303
      if(abs(yik).gt.rc) goto 303
      if(abs(zik).gt.rc) goto 303
      rik2=xik*xik+yik*yik+zik*zik
      if(rik2.gt.rc2) goto 303
      if(rik2.lt.0.0000001d0) goto 303
      rjk2=xjk*xjk+yjk*yjk+zjk*zjk
      if(rjk2.lt.0.0000001d0) goto 303
      rik=dsqrt(rik2)
      rjk=dsqrt(rjk2)
      rrik=rik/para(6)
      rrjk=rjk/para(6)
!      print *,'hit'
      temp=1.0d0/(rrij-para(3))

      temp=temp+1.0d0/(rrik-para(3))

      temp=para(4)*dexp(para(5)*temp)
      temp2=rrij**2+rrik**2-rrjk**2
      temp2=temp2/(2.0d0*rrij*rrik)
      htemp=temp*2.0d0*(temp2+1.0d0/3.0d0)
      h=temp*(temp2+1.0d0/3.0d0)**2
!      epot=epot+h*para(7)/2.0d0 !2.0d0 ! Energy
      epot=h*para(7) ! Energy

!c--------------------------------FORCE-----------------------------
      if(i.ne.j) then
         dtdr=2.0d0*rrij**2*rrik
         temp=(rrij**2+rrik**2-rrjk**2)
         dtdr=dtdr-temp*rrik
         dtdr=dtdr/2.0d0/(rrij*rrik)**2
         dhdr=-h*para(5)*(rrij-para(3))**(-2)
         temp=htemp*dtdr
         dhdr=dhdr+temp
         dhdr=-dhdr*para(7)/2.0d0     !/2.0d0
         fx(i)=fx(i)-dhdr*xij/rij/para(6)*1.0d9
         fy(i)=fy(i)-dhdr*yij/rij/para(6)*1.0d9
         fz(i)=fz(i)-dhdr*zij/rij/para(6)*1.0d9
         fx(j)=fx(j)+dhdr*xij/rij/para(6)*1.0d9
         fy(j)=fy(j)+dhdr*yij/rij/para(6)*1.0d9
         fz(j)=fz(j)+dhdr*zij/rij/para(6)*1.0d9

      endif
      if(i.ne.k) then
         dtdr=2.0d0*rrij*rrik**2
         temp=(rrij**2+rrik**2-rrjk**2)
         dtdr=dtdr-temp*rrij
         dtdr=dtdr/2.0d0/(rrij*rrik)**2
         dhdr=-h*para(5)*(rrik-para(3))**(-2)
         temp=htemp*dtdr
         dhdr=dhdr+temp
         dhdr=-dhdr*para(7)/2.0d0     !/2.0d0
         fx(i)=fx(i)-dhdr*xik/rik/para(6)*1.0d9
         fy(i)=fy(i)-dhdr*yik/rik/para(6)*1.0d9
         fz(i)=fz(i)-dhdr*zik/rik/para(6)*1.0d9
         fx(k)=fx(k)+dhdr*xik/rik/para(6)*1.0d9
         fy(k)=fy(k)+dhdr*yik/rik/para(6)*1.0d9
         fz(k)=fz(k)+dhdr*zik/rik/para(6)*1.0d9
      endif
      if(j.ne.k) then
         dtdr=-rrjk/rrij/rrik
         dhdr=htemp*dtdr
         dhdr=-dhdr*para(7)/2.0d0 !/2.0d0
         fx(j)=fx(j)-dhdr*xjk/rjk/para(6)*1.0d9
         fy(j)=fy(j)-dhdr*yjk/rjk/para(6)*1.0d9
         fz(j)=fz(j)-dhdr*zjk/rjk/para(6)*1.0d9
         fx(k)=fx(k)+dhdr*xjk/rjk/para(6)*1.0d9
         fy(k)=fy(k)+dhdr*yjk/rjk/para(6)*1.0d9
         fz(k)=fz(k)+dhdr*zjk/rjk/para(6)*1.0d9
      endif


 303  continue

      return
      end
