!
!   Tersoff potential
!   for Multisystem (Silicon, Cargon, Germanium)
!   with modification by Belko et al.
!   calculation of force, total energy, stress
!
!     Modified 17/06/06, now not using matrices zmat(i,j) and b(i,j)

      module tersoff_belko_nomat_function
!      double precision, allocatable :: zmat(:,:),b(:,:)
      double precision :: terrrm(3), terssm(3), teraam(3), terbbm(3), terlambdam(3), termum(3), terbetam(3)
      double precision :: ternm(3), tercm(3), terdm(3), terhm(3), terwm(3)
      double precision :: terrlm(3), terrhm(3), teraapm(3), terlambdapm(3)
      double precision :: terbbpm(3), termupm(3), terhfm(3)
      integer :: termm(3)
      double precision :: terkaim(3,3)
      double precision :: terrr, terss, teraa, terbb, terlambda, termu, terbeta
      double precision :: tern, terc, terd, terh, terw
      integer :: term
      double precision :: terkai
      end module

      subroutine tersoff_belko_nomat
      use tersoff_belko_nomat_function
      use variables
      use constants
      use bookkeep_param
      use debug_variables
      implicit double precision (a-h,o-z)
      double precision :: ki
      integer :: istat
      integer :: tsp
!      if (.not.allocated(zmat)) allocate(zmat(n,n))
!      if (.not.allocated(b)) allocate(b(n,n))
      if (bookkeep_algo /= 2) stop 'Error in tersoff.F: Bookkeep algo is not DOUBLE'

!--tersoff
      pi=4.0d0*atan(1.0d0)
      eps=1.0d-20
      ev2j=1.6021892d-19        !eV -> J
      j2ev=1.0d0/ev2j           !J -> eV
      mode=2                    !1:38-14(B,C),2:39-8(Multi),1:Date's type
!      mparam=0                  !0:Si(B), 1:Si(C) [see PRB 38 p.9902]
!                                !4:Date's potential(B),5:Yasukawa,6:Date's Potential(read from file)
!      if (TRIM(potential_arg)=='B') mparam=0
!      if (TRIM(potential_arg)=='C') mparam=1

!CCC  PARAMETER FROM Phys.Rev.B 39-8(1989)p.5566
!     FOR SILICON
      terrrm(1)=     2.70d-10
      terssm(1)=     3.00d-10
      teraam(1)=     1.8308d3*ev2j
      terbbm(1)=     4.7118d2*ev2j
      terlambdam(1)= 2.4799d10
      termum(1)=     1.7322d10
      terbetam(1)=   1.0999d-6
      ternm(1)=      7.8734d-1
      termm(1)=      3
      tercm(1)=      1.0039d5
      terdm(1)=      1.6218d1
      terhm(1)=     -5.9826d-1
      terwm(1)=      1.0d0

      terrlm(1)=     1.1972d-10
      terrhm(1)=     1.8153d-10
      teraapm(1)=    854.43d0*ev2j
      terlambdapm(1)=2.7623d0
      terbbpm(1)=    6.6496d0*ev2j
      termupm(1)=    0.08727d0
      terhfm(1)=     2.4829d10

!     FOR CARBON
      terrrm(2)=     1.80d-10
      terssm(2)=     2.10d-10
      teraam(2)=     1.3936d3*ev2j
      terbbm(2)=     3.467d2*ev2j
      terlambdam(2)= 3.4879d10
      termum(2)=     2.2119d10
      terbetam(2)=   1.5724d-7
      ternm(2)=      7.2751d-1
      termm(2)=      3
      tercm(2)=      3.8049d4
      terdm(2)=      4.384d0
      terhm(2)=     -5.7058d-1
      terwm(2)=      1.0d0

      terrlm(2)=     0.50000d-10
      terrhm(2)=     1.0902d-10
      teraapm(2)=    1502.0d0*ev2j
      terlambdapm(2)=5.0025d0
      terbbpm(2)=    1.7967d0*ev2j
      termupm(2)=   -1.1695d0
      terhfm(2)=     3.1105d10

!     FOR Si-C
      terrlm(3)=     0.75593d-10
      terrhm(3)=     1.3581d-10
      teraapm(3)=    1238.3d0*ev2j
      terlambdapm(3)=3.3748d0
      terbbpm(3)=    60.776d0*ev2j
      termupm(3)=    1.1553d0
      terhfm(3)=     2.9064d10

!     FOR INTER-ATOM TERM
      terkaim=0.0d0
      terkaim(1,1)=1.0d0
      terkaim(2,2)=1.0d0
      terkaim(1,2)=0.9776d0
      terkaim(2,1)=0.9776d0
!
      terw=1.0d0
!--

      istat=1

      rc=max(terssm(1),terssm(2),terssm(3))
      rc2=terss**2

!   Virial term reset
      virx=0.0d0; viry=0.0d0; virz=0.0d0
      dmat=0.0d0
      satom=0.0d0               ! Atomic stress


!   Loop
      enpot=0.0d0
      enpoteach=0.0d0
      fx(1:n)=0.0d0
      fy(1:n)=0.0d0
      fz(1:n)=0.0d0
      kns=0
      iloop: do i=1,n
         rix=rx(i)
         riy=ry(i)
         riz=rz(i)
         kn=knum(i)
         if (kn.eq.0) cycle
         kns0=kns
         do 200 jcount=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j); yy=ry(i)-ry(j); zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            rjx=rx(i)-xx; rjy=ry(i)-yy; rjz=rz(i)-zz
               rij2=(rjx-rix)**2+(rjy-riy)**2+(rjz-riz)**2
               if (rij2.le.eps) go to 201
               rij=sqrt(rij2)
               terrr=sqrt(terrrm(tsp(asp(i)))*terrrm(tsp(asp(j))))
               terss=sqrt(terssm(tsp(asp(i)))*terssm(tsp(asp(j))))
               fij=fcut(rij,terrr,terss)
               dfij=fcutd(rij,terrr,terss)
               if (fij.le.eps) go to 201
               terbeta=terbetam(tsp(asp(i)))
               tern=ternm(tsp(asp(i)))
               terkai=terkaim(tsp(asp(i)),tsp(asp(j)))
               zmatij=zmatm(i,j,rx(i)-rjx,ry(i)-rjy,rz(i)-rjz)
               bij=terkai*(1.0d0+(terbeta*zmatij)**tern)**(-0.5d0/tern)
               terbeta=terbetam(tsp(asp(j)))
               tern=ternm(tsp(asp(j)))
               terkai=terkaim(tsp(asp(j)),tsp(asp(i)))
               zmatji=zmatm(j,i,-rx(i)+rjx,-ry(i)+rjy,-rz(i)+rjz)
               bji=terkai*(1.0d0+(terbeta*zmatji)**tern)**(-0.5d0/tern)
               go to 1111
               if ((tsp(asp(i))==1).and.(tsp(asp(j))==1)) then
                  if (rij>1.8153d-10) then
                     teraa=sqrt(teraam(tsp(asp(i)))*teraam(tsp(asp(j))))
                     terbb=sqrt(terbbm(tsp(asp(i)))*terbbm(tsp(asp(j))))
                     termu=(termum(tsp(asp(i)))+termum(tsp(asp(j))))/2.0d0
                     terlambda=(terlambdam(tsp(asp(i)))+terlambdam(tsp(asp(j))))/2.0d0
                  else if (rij>1.1972d-10) then
                     teraa=teraapm(tsp(asp(i)))
                     terbb=terbbpm(tsp(asp(i)))
                     termu=termupm(tsp(asp(i)))
                     terlambda=terlambdapm(tsp(asp(i)))
                  else
                     print *,'ZBL is required'
                     stop
                  endif
               else if ((tsp(asp(i))==2).and.(tsp(asp(j))==2)) then
                  if (rij>1.0902d-10) then
                     teraa=sqrt(teraam(tsp(asp(i)))*teraam(tsp(asp(j))))
                     terbb=sqrt(terbbm(tsp(asp(i)))*terbbm(tsp(asp(j))))
                     termu=(termum(tsp(asp(i)))+termum(tsp(asp(j))))/2.0d0
                     terlambda=(terlambdam(tsp(asp(i)))+terlambdam(tsp(asp(j))))/2.0d0
                  else if (rij>0.5000d-10) then
                     teraa=teraapm(tsp(asp(i)))
                     terbb=terbbpm(tsp(asp(i)))
                     termu=termupm(tsp(asp(i)))
                     terlambda=terlambdapm(tsp(asp(i)))
                  else
                     print *,'ZBL is required'
                     stop
                  endif
               else
                  if (rij>1.3581d-10) then
                     teraa=sqrt(teraam(tsp(asp(i)))*teraam(tsp(asp(j))))
                     terbb=sqrt(terbbm(tsp(asp(i)))*terbbm(tsp(asp(j))))
                     termu=(termum(tsp(asp(i)))+termum(tsp(asp(j))))/2.0d0
                     terlambda=(terlambdam(tsp(asp(i)))+terlambdam(tsp(asp(j))))/2.0d0
                  else if (rij>0.75593d-10) then
                     teraa=teraapm(3)
                     terbb=terbbpm(3)
                     termu=termupm(3)
                     terlambda=terlambdapm(3)
                  else
                     print *,'ZBL is required'
                     stop
                  endif
               endif
1111           continue
               teraa=sqrt(teraam(tsp(asp(i)))*teraam(tsp(asp(j))))
               terbb=sqrt(terbbm(tsp(asp(i)))*terbbm(tsp(asp(j))))
               termu=(termum(tsp(asp(i)))+termum(tsp(asp(j))))/2.0d0
               terlambda=(terlambdam(tsp(asp(i)))+terlambdam(tsp(asp(j))))/2.0d0
               terkai=terkaim(tsp(asp(i)),tsp(asp(j)))
               ushtij=-fij*terkai*bij*terbb*exp(-termu*rij)
               urepij=fij*teraa*exp(-terlambda*rij)
               uij=urepij+ushtij
               enpoteach(i)=enpoteach(i)+uij*0.5d0
!!     Atomic stress
!               ad1=0.5d0*(-terlambda)*urepij/rij !! bug fixed 150606
               ad1=0.5d0*( (-terlambda)*urepij + dfij/fij*urepij )/rij
               adx=rjx-rix; ady=rjy-riy; adz=rjz-riz
               satom(i,1,1)=satom(i,1,1)+ad1*adx*adx
               satom(i,1,2)=satom(i,1,2)+ad1*ady*adx
               satom(i,2,2)=satom(i,2,2)+ad1*ady*ady
               satom(i,1,3)=satom(i,1,3)+ad1*adz*adx
               satom(i,2,3)=satom(i,2,3)+ad1*adz*ady
               satom(i,3,3)=satom(i,3,3)+ad1*adz*adz
!--------------
               ffx=0.0d0;ffy=0.0d0;ffz=0.0d0
               knsk=kns0
               do 300 kcount=1,kn
                  knsk=knsk+1
                  k=idpt(knsk,1)
                  ix=idpt(knsk,2); iy=idpt(knsk,3); iz=idpt(knsk,4)
                  xx=rx(i)-rx(k); yy=ry(i)-ry(k); zz=rz(i)-rz(k)
                  call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
                  rkx=rx(i)-xx; rky=ry(i)-yy; rkz=rz(i)-zz
                     rik2=(rkx-rix)**2+(rky-riy)**2+(rkz-riz)**2
                     if (rik2.le.eps) go to 301
                     rik=sqrt(rik2)
                     terrr=sqrt(terrrm(tsp(asp(i)))*terrrm(tsp(asp(k))))
                     terss=sqrt(terssm(tsp(asp(i)))*terssm(tsp(asp(k))))
                     fik=fcut(rik,terrr,terss)
                     if (fik.le.eps) go to 301
                     rjk2=(rkx-rjx)**2+(rky-rjy)**2+(rkz-rjz)**2
                     if (rjk2.le.eps) go to 301
                     rjk=sqrt(rjk2)
                     costh=(rik2+rij2-rjk2)/(2.0d0*rik*rij)
                     terc=tercm(tsp(asp(i)));terd=terdm(tsp(asp(i)));terh=terhm(tsp(asp(i)))
                     g=(1.0d0+terc**2/terd**2 &
                     -terc**2/(terd**2+(terh-costh)**2))
                     terbb=sqrt(terbbm(tsp(asp(i)))*terbbm(tsp(asp(j))))
                     termu=(termum(tsp(asp(i)))+termum(tsp(asp(j))))/2.0d0
                     terbeta=terbetam(tsp(asp(i)))
                     tern=ternm(tsp(asp(i)))
                     terkai=terkaim(tsp(asp(i)),tsp(asp(j)))
                     dvdz=terbb*fij*exp(-termu*rij)*terkai*bij*terbeta &
                     **tern*zmatij**tern/2.0d0/(1.0d0+(terbeta*zmatij) &
                     **tern)/zmatij

!                     if (mode==1) then
!                        dzdr1=dble(term)*termu**term &
!                        *(rij-rik)**(term-1) &
!                        *fik*exp(termu**term*(rij-rik)**term) &
!                        *(1.0d0+terc**2/terd**2 &
!                        -terc**2/(terd**2+(terh-costh)**2))
!                        dzdc=fik*exp(termu**term*(rij-rik)**term) &
!                        *(-2.0d0*terc**2*(terh-costh)) &
!                        /((terd**2+(terh-costh)**2)**2)
!                     else
                        dzdr1=0.0d0
                        dzdc=fik*terw &
                        *(-2.0d0*terc**2*(terh-costh)) &
                        /((terd**2+(terh-costh)**2)**2)
!                     endif
                     terrr=sqrt(terrrm(tsp(asp(i)))*terrrm(tsp(asp(k))))
                     terss=sqrt(terssm(tsp(asp(i)))*terssm(tsp(asp(k))))
!                     dzdr2=fcutd(rik,terrr,terss)-dzdr1 !! bug fixed 150606
!                     dzdr2=fcutd(rik,terrr,terss)*exp(termu**term*(rij-rik)**term) & !!! fixed 170606
                     dzdr2=fcutd(rik,terrr,terss)*terw &
                        *(1.0d0+terc**2/terd**2 &
                        -terc**2/(terd**2+(terh-costh)**2))-dzdr1
!                     dcdrx=(rik*costh-rij)/(rik*rij2)*(rjx-rix) &
!                     +(rij*costh-rik)/(rij*rik2)*(rkx-rix)
!                     dcdry=(rik*costh-rij)/(rik*rij2)*(rjy-riy) &
!                     +(rij*costh-rik)/(rij*rik2)*(rky-riy)
!                     dcdrz=(rik*costh-rij)/(rik*rij2)*(rjz-riz) &
!                     +(rij*costh-rik)/(rij*rik2)*(rkz-riz)
                     dxij=rix-rjx;dyij=riy-rjy;dzij=riz-rjz
                     dxki=rkx-rix;dyki=rky-riy;dzki=rkz-riz
                     dcdrx=1.0d0/rik*(dxij/rij+dxki*costh/rik) &
                     -1.0d0/rij*(dxki/rik+dxij*costh/rij)
                     dcdry=1.0d0/rik*(dyij/rij+dyki*costh/rik) &
                     -1.0d0/rij*(dyki/rik+dyij*costh/rij)
                     dcdrz=1.0d0/rik*(dzij/rij+dzki*costh/rik) &
                     -1.0d0/rij*(dzki/rik+dzij*costh/rij)
                     ffx=ffx-dvdz*(dzdr1*dxij/rij-dzdr2*dxki/rik+dzdc*dcdrx)/2.0d0
                     ffy=ffy-dvdz*(dzdr1*dyij/rij-dzdr2*dyki/rik+dzdc*dcdry)/2.0d0
                     ffz=ffz-dvdz*(dzdr1*dzij/rij-dzdr2*dzki/rik+dzdc*dcdrz)/2.0d0
!!     Atomic stress
!                     ad1=0.5d0*fij*terbb*exp(-termu*rij)
                     ad1=0.5d0*dvdz*dzdr1/rij
                     ad2=0.5d0*dvdz*dzdr2/rik
!                     adx=rjx-rix; ady=rjy-riy; adz=rjz-riz
                     satom(i,1,1)=satom(i,1,1)+ad1*dxij*dxij+ad2*dxki*dxki
                     satom(i,1,2)=satom(i,1,2)+ad1*dyij*dxij+ad2*dyki*dxki
                     satom(i,2,2)=satom(i,2,2)+ad1*dyij*dyij+ad2*dyki*dyki
                     satom(i,1,3)=satom(i,1,3)+ad1*dzij*dxij+ad2*dzki*dxki
                     satom(i,2,3)=satom(i,2,3)+ad1*dzij*dyij+ad2*dzki*dyki
                     satom(i,3,3)=satom(i,3,3)+ad1*dzij*dzij+ad2*dzki*dzki

                     dxjk=rkx-rjx;dyjk=rky-rjy;dzjk=rkz-rjz
                     ad0=0.5d0*dvdz*dzdc
                     ad1=-1.0d0/rik/rij
                     ad2=1.0d0/rik/rij-costh/rik**2
                     ad3=1.0d0/rik/rij-costh/rij**2
                     satom(i,1,1)=satom(i,1,1)+ad0*(ad1*dxjk*dxjk+ad2*dxki*dxki+ad3*dxij*dxij)
                     satom(i,1,2)=satom(i,1,2)+ad0*(ad1*dyjk*dxjk+ad2*dyki*dxki+ad3*dyij*dxij)
                     satom(i,2,2)=satom(i,2,2)+ad0*(ad1*dyjk*dyjk+ad2*dyki*dyki+ad3*dyij*dyij)
                     satom(i,1,3)=satom(i,1,3)+ad0*(ad1*dzjk*dxjk+ad2*dzki*dxki+ad3*dzij*dxij)
                     satom(i,2,3)=satom(i,2,3)+ad0*(ad1*dzjk*dyjk+ad2*dzki*dyki+ad3*dzij*dyij)
                     satom(i,3,3)=satom(i,3,3)+ad0*(ad1*dzjk*dzjk+ad2*dzki*dzki+ad3*dzij*dzij)
 301                 continue
 300           continue
!--------------
!               ad1=(dfij-terlambda*fij)*teraa*exp(-terlambda*rij)
!               fx(i)=fx(i)+ad1*(rjx-rix)/rij
!               fy(i)=fy(i)+ad1*(rjy-riy)/rij
!               fz(i)=fz(i)+ad1*(rjz-riz)/rij
!               ad1=(fij*termu-dfij)*terbb*(b(i,j)+b(j,i))/2.0d0*exp(-termu*rij)  !dvdr1
!               fx(i)=fx(i)+ad1*(rjx-rix)/rij
!               fy(i)=fy(i)+ad1*(rjy-riy)/rij
!               fz(i)=fz(i)+ad1*(rjz-riz)/rij
               go to 2222
               if ((tsp(asp(i))==1).and.(tsp(asp(j))==1)) then
                  if (rij>1.8153d-10) then
                     teraa=sqrt(teraam(tsp(asp(i)))*teraam(tsp(asp(j))))
                     terbb=sqrt(terbbm(tsp(asp(i)))*terbbm(tsp(asp(j))))
                     termu=(termum(tsp(asp(i)))+termum(tsp(asp(j))))/2.0d0
                     terlambda=(terlambdam(tsp(asp(i)))+terlambdam(tsp(asp(j))))/2.0d0
                  else if (rij>1.1972d-10) then
                     teraa=teraapm(tsp(asp(i)))
                     terbb=terbbpm(tsp(asp(i)))
                     termu=termupm(tsp(asp(i)))
                     terlambda=terlambdapm(tsp(asp(i)))
                  else
                     print *,'ZBL is required'
                     stop
                  endif
               else if ((tsp(asp(i))==2).and.(tsp(asp(j))==2)) then
                  if (rij>1.0902d-10) then
                     teraa=sqrt(teraam(tsp(asp(i)))*teraam(tsp(asp(j))))
                     terbb=sqrt(terbbm(tsp(asp(i)))*terbbm(tsp(asp(j))))
                     termu=(termum(tsp(asp(i)))+termum(tsp(asp(j))))/2.0d0
                     terlambda=(terlambdam(tsp(asp(i)))+terlambdam(tsp(asp(j))))/2.0d0
                  else if (rij>0.5000d-10) then
                     teraa=teraapm(tsp(asp(i)))
                     terbb=terbbpm(tsp(asp(i)))
                     termu=termupm(tsp(asp(i)))
                     terlambda=terlambdapm(tsp(asp(i)))
                  else
                     print *,'ZBL is required'
                     stop
                  endif
               else
                  if (rij>1.3581d-10) then
                     teraa=sqrt(teraam(tsp(asp(i)))*teraam(tsp(asp(j))))
                     terbb=sqrt(terbbm(tsp(asp(i)))*terbbm(tsp(asp(j))))
                     termu=(termum(tsp(asp(i)))+termum(tsp(asp(j))))/2.0d0
                     terlambda=(terlambdam(tsp(asp(i)))+terlambdam(tsp(asp(j))))/2.0d0
                  else if (rij>0.75593d-10) then
                     teraa=teraapm(3)
                     terbb=terbbpm(3)
                     termu=termupm(3)
                     terlambda=terlambdapm(3)
                  else
                     print *,'ZBL is required'
                     stop
                  endif
               endif
2222           continue
               teraa=sqrt(teraam(tsp(asp(i)))*teraam(tsp(asp(j))))
               terbb=sqrt(terbbm(tsp(asp(i)))*terbbm(tsp(asp(j))))
               termu=(termum(tsp(asp(i)))+termum(tsp(asp(j))))/2.0d0
               terlambda=(terlambdam(tsp(asp(i)))+terlambdam(tsp(asp(j))))/2.0d0
               terkai=terkaim(tsp(asp(i)),tsp(asp(j)))
               dvdr1=teraa*exp(-terlambda*rij)*(dfij-terlambda &
               *fij)-terkai*bij*terbb*exp(-termu*rij) &
               *(dfij-termu*fij)
               dvdr2=teraa*exp(-terlambda*rij)*(dfij-terlambda &
               *fij)-terkai*bji*terbb*exp(-termu*rij) &
               *(dfij-termu*fij)
               fx(i)=fx(i)-((dvdr1+dvdr2)*dxij/rij)/2.0d0
               fy(i)=fy(i)-((dvdr1+dvdr2)*dyij/rij)/2.0d0
               fz(i)=fz(i)-((dvdr1+dvdr2)*dzij/rij)/2.0d0
               
               fx(i)=fx(i)+ffx
               fy(i)=fy(i)+ffy
               fz(i)=fz(i)+ffz
!!     Atomic stress
               terkai=terkaim(tsp(asp(i)),tsp(asp(j)))
!               ad1=0.5d0*fij*terbb*terkai*(b(i,j)+b(j,i))/2.0d0 & !! bug fixed 150606
!               *(termu)*exp(-termu*rij)/rij
               ad1=0.5d0*(fij*termu-dfij)*terbb*terkai*(bij+bji)/2.0d0*exp(-termu*rij)/rij
               adx=rjx-rix; ady=rjy-riy; adz=rjz-riz
               satom(i,1,1)=satom(i,1,1)+ad1*adx*adx
               satom(i,1,2)=satom(i,1,2)+ad1*ady*adx
               satom(i,2,2)=satom(i,2,2)+ad1*ady*ady
               satom(i,1,3)=satom(i,1,3)+ad1*adz*adx
               satom(i,2,3)=satom(i,2,3)+ad1*adz*ady
               satom(i,3,3)=satom(i,3,3)+ad1*adz*adz
 201           continue
 200     continue
      end do iloop
      print *,fx
!cccc u_ji/r_i
      kns=0
      iloop2: do i=1,n
         rix=rx(i)
         riy=ry(i)
         riz=rz(i)
         kn=knum(i)
         if (kn.eq.0) cycle
         kns0=kns
         do 2200 jcount=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j); yy=ry(i)-ry(j); zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            rjx=rx(i)-xx; rjy=ry(i)-yy; rjz=rz(i)-zz
               rij2=(rjx-rix)**2+(rjy-riy)**2+(rjz-riz)**2
               if (rij2.le.eps) go to 2201
               rij=sqrt(rij2)
               terrr=sqrt(terrrm(tsp(asp(i)))*terrrm(tsp(asp(j))))
               terss=sqrt(terssm(tsp(asp(i)))*terssm(tsp(asp(j))))
               fij=fcut(rij,terrr,terss)
               dfij=fcutd(rij,terrr,terss)  !!! added 160606
               if (fij.le.eps) go to 2201
               terbeta=terbetam(tsp(asp(j)))
               tern=ternm(tsp(asp(j)))
               terkai=terkaim(tsp(asp(j)),tsp(asp(i)))
               zmatji=zmatm(j,i,-xx,-yy,-zz)
               bji=terkai*(1.0d0+(terbeta*zmatji)**tern)**(-0.5d0/tern)
!--------------
               ffx=0.0d0;ffy=0.0d0;ffz=0.0d0
               kjn=knum(j)
               knsk=0; if (j>0) then; do kcount=1,j-1; knsk=knsk+knum(kcount); enddo; endif
               do 2300 kcount=1,kjn
                  knsk=knsk+1
                  k=idpt(knsk,1)
                  ix=idpt(knsk,2); iy=idpt(knsk,3); iz=idpt(knsk,4)
                  xx=rx(j)-rx(k); yy=ry(j)-ry(k); zz=rz(j)-rz(k)
                  call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)


!!! N.B. "replica" k-coordinates must be calculated from "replica" j-coordinates,
!!!      not "original" j-coordinates.
!!!      i.e. "rkx=rx(j)-xx; rky=ry(j)-yy; rkz=rz(j)-zz" is wrong.
                  rkx=rjx-xx; rky=rjy-yy; rkz=rjz-zz
                     rik2=(rkx-rix)**2+(rky-riy)**2+(rkz-riz)**2
                     if (rik2.le.eps) go to 2301
                     rik=sqrt(rik2)
                     rjk2=(rkx-rjx)**2 &
                         +(rky-rjy)**2+(rkz-rjz)**2
                     if (rjk2.le.eps) go to 2301
                     rjk=sqrt(rjk2)
                     terrr=sqrt(terrrm(tsp(asp(j)))*terrrm(tsp(asp(k))))
                     terss=sqrt(terssm(tsp(asp(j)))*terssm(tsp(asp(k))))
                     fjk=fcut(rjk,terrr,terss)
                     if (fjk.le.eps) go to 2301
                     rjk=sqrt(rjk2)
!u_ij/r_i
                     costh=(rjk2+rij2-rik2)/(2.0d0*rjk*rij)
                     terc=tercm(tsp(asp(j)));terd=terdm(tsp(asp(j)));terh=terhm(tsp(asp(j)))
                     g=(1.0d0+terc**2/terd**2 &
                     -terc**2/(terd**2+(terh-costh)**2))
                     terrr=sqrt(terrrm(tsp(asp(j)))*terrrm(tsp(asp(i))))
                     terss=sqrt(terssm(tsp(asp(j)))*terssm(tsp(asp(i))))
                     fij=fcut(rij,terrr,terss)
                     terbb=sqrt(terbbm(tsp(asp(j)))*terbbm(tsp(asp(i))))
                     termu=(termum(tsp(asp(j)))+termum(tsp(asp(i))))/2.0d0
                     terbeta=terbetam(tsp(asp(j)))
                     tern=ternm(tsp(asp(j)))
                     terkai=terkaim(tsp(asp(j)),tsp(asp(i)))
                     dvdz=terbb*fij*exp(-termu*rij)*terkai*bji*terbeta &
                     **tern*zmatji**tern/2.0d0/(1.0d0+(terbeta*zmatji) &
                     **tern)/zmatji
!                     if (mode==1) then
!                        dzdr=termu**term*dble(term) &
!                        *(rij-rjk)**(term-1) &
!                        *fjk*exp(termu**term*(rij-rjk)**term) &
!                        *(1.0d0+terc**2/terd**2 &
!                        -terc**2/(terd**2+(terh-costh)**2))
!                        dzdc=fjk*exp(termu**term*(rij-rjk)**term) &
!                        *(-2.0d0*terc**2*(terh-costh)) &
!                        /((terd**2+(terh-costh)**2)**2)
!                     else
                        dzdr=0.0d0
                        dzdc=fjk*terw &
                        *(-2.0d0*terc**2*(terh-costh)) &
                        /((terd**2+(terh-costh)**2)**2)
!                     endif
                     dcdrx=(rjk2-rik2-rij2)/(2.0d0*rij2*rij*rjk) &
                     *(rjx-rix)+1.0d0/(rij*rjk)*(rkx-rix)
                     dcdry=(rjk2-rik2-rij2)/(2.0d0*rij2*rij*rjk) &
                     *(rjy-riy)+1.0d0/(rij*rjk)*(rky-riy)
                     dcdrz=(rjk2-rik2-rij2)/(2.0d0*rij2*rij*rjk) &
                     *(rjz-riz)+1.0d0/(rij*rjk)*(rkz-riz)
                     dxij=rix-rjx;dyij=riy-rjy;dzij=riz-rjz
!                     dxjk=rjx-rkx;dyjk=rjy-rky;dzjk=rjz-rkz
!                     dcdrx=-1.0d0/rij*(dxjk/rjk+dxij*costh/rij)
!                     dcdry=-1.0d0/rij*(dyjk/rjk+dyij*costh/rij)
!                     dcdrz=-1.0d0/rij*(dzjk/rjk+dzij*costh/rij)
                     ffx=ffx-dvdz*(dzdr*dxij/rij+dzdc*dcdrx)/2.0d0
                     ffy=ffy-dvdz*(dzdr*dyij/rij+dzdc*dcdry)/2.0d0
                     ffz=ffz-dvdz*(dzdr*dzij/rij+dzdc*dcdrz)/2.0d0
!u_jk/r_i
                     terbb=sqrt(terbbm(tsp(asp(j)))*terbbm(tsp(asp(k))))
                     termu=(termum(tsp(asp(j)))+termum(tsp(asp(k))))/2.0d0
                     terbeta=terbetam(tsp(asp(j)))
                     tern=ternm(tsp(asp(j)))
                     terkai=terkaim(tsp(asp(j)),tsp(asp(k)))
                     zmatjk=zmatm(j,k,rjx-rkx,rjy-rky,rjz-rkz)
                     bjk=terkai*(1.0d0+(terbeta*zmatjk)**tern)**(-0.5d0/tern)
                     dvdz=terbb*fjk*exp(-termu*rjk)*terkai*bjk*terbeta &
                     **tern*zmatjk**tern/2.0d0/(1.0d0+(terbeta*zmatjk) &
                     **tern)/zmatjk
!                     if (mode==1) then
!                        dzdr=-dble(term)*termu**term &
!                        *(rjk-rij)**(term-1)*fij*g*exp(termu**term &
!                        *(rjk-rij)**term)
!                        dzdr=-dble(term)*termu**term &
!                        *(rjk-rij)**(term-1)*fij*g*exp(termu**term & !!! amended 160606
!                        *(rjk-rij)**term)+dfij*exp(termu**term*(rjk-rij)**term)*g
!                        dzdc=fij*exp(termu**term*(rjk-rij)**term) &
!                        *(-2.0d0*terc**2.0*(terh-costh)) &
!                        /(terd**2+(terh-costh)**2)**2
!                     else
!                        dzdr=0.0d0 !!! bug fixed 170606
                        dzdr=dfij*terw*g
                        dzdc=fij*terw &
                        *(-2.0d0*terc**2.0*(terh-costh)) &
                        /(terd**2+(terh-costh)**2)**2
!                     endif
                     ad5=(rjk2-rik2-rij2)/2.0d0/rij2/rij/rjk
                     ad6=1.0d0/rij/rjk
                     dcdrx=(ad5*(rjx-rix)+ad6*(rkx-rix))
                     dcdry=(ad5*(rjy-riy)+ad6*(rky-riy))
                     dcdrz=(ad5*(rjz-riz)+ad6*(rkz-riz))
                     dxij=rix-rjx;dyij=riy-rjy;dzij=riz-rjz
!                     dxjk=rjx-rkx;dyjk=rjy-rky;dzjk=rjz-rkz
!                     dcdrx=-1.0d0/rij*(dxjk/rjk+dxij*costh/rij)
!                     dcdry=-1.0d0/rij*(dyjk/rjk+dyij*costh/rij)
!                     dcdrz=-1.0d0/rij*(dzjk/rjk+dzij*costh/rij)
                     ffx=ffx-dvdz*(dzdr*dxij/rij+dzdc*dcdrx)/2.0d0
                     ffy=ffy-dvdz*(dzdr*dyij/rij+dzdc*dcdry)/2.0d0
                     ffz=ffz-dvdz*(dzdr*dzij/rij+dzdc*dcdrz)/2.0d0
 2301                continue
 2300          continue
               fx(i)=fx(i)+ffx
               fy(i)=fy(i)+ffy
               fz(i)=fz(i)+ffz
!--------------
 2201          continue
 2200    continue
! 2100 continue
      end do iloop2

!---Etot---
      enpot=0.0d0
      do i=1,n
         enpot=enpot+enpoteach(i)
      enddo

!!     Atomic stress
      do i=1,n
         satom(i,2,1)=satom(i,1,2)
         satom(i,3,1)=satom(i,1,3)
         satom(i,3,2)=satom(i,2,3)
      enddo
      dmat=0.0d0
! The sign of dmat is opposite to stress tensor's (dmat is the "virial" tensor)
      do i=1,n
         dmat(1:3,1:3)=dmat(1:3,1:3)-satom(i,1:3,1:3)
      enddo
      virx=dmat(1,1); viry=dmat(2,2); virz=dmat(3,3)

      satom=satom*dble(n)/v

      end

!      integer function tsp(species)
!      character(2) :: species
!      if (species=='Si') tsp=1
!      if (species=='C ') tsp=2
!      if (species=='Ge') tsp=3
!      end

!      double precision function zmatm(i,j,xxx,yyy,zzz)
!      use tersoff_multi_nomat_function
!      use bookkeep_param
!      use variables
!      use constants
!      implicit double precision (a-h,o-z)
!      double precision :: xx,yy,zz,eps,xxx,yyy,zzz
!      integer :: tsp
!      eps=1.0d-20
!      zmatm=0.0d0
!      rix=rx(i)
!      riy=ry(i)
!      riz=rz(i)
!      rjx=rx(i)-xxx; rjy=ry(i)-yyy; rjz=rz(i)-zzz
!      rij2=xxx**2+yyy**2+zzz**2
!      if (rij2.le.eps) return
!      rij=sqrt(rij2)
!      terrr=sqrt(terrrm(tsp(asp(i)))*terrrm(tsp(asp(j))))
!      terss=sqrt(terssm(tsp(asp(i)))*terssm(tsp(asp(j))))
!      fij=fcut(rij,terrr,terss)
!!      if (fij.le.eps) return
!      kn=knum(i)
!      knsk=0; if (i>0) then; do ii=1,i-1; knsk=knsk+knum(ii); enddo; endif
!      do 1300 kcount=1,kn
!         knsk=knsk+1
!         k=idpt(knsk,1)
!         ix=idpt(knsk,2); iy=idpt(knsk,3); iz=idpt(knsk,4)
!         xx=rx(i)-rx(k); yy=ry(i)-ry(k); zz=rz(i)-rz(k)
!         call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
!         rkx=rx(i)-xx; rky=ry(i)-yy; rkz=rz(i)-zz
!         rik2=(rkx-rix)**2+(rky-riy)**2+(rkz-riz)**2
!         if (rik2.le.eps) go to 1301
!         rik=sqrt(rik2)
!         terrr=sqrt(terrrm(tsp(asp(i)))*terrrm(tsp(asp(k))))
!         terss=sqrt(terssm(tsp(asp(i)))*terssm(tsp(asp(k))))
!         fik=fcut(rik,terrr,terss)
!         if (fik.le.eps) go to 1301
!         rjk2=(rkx-rjx)**2+(rky-rjy)**2+(rkz-rjz)**2
!         if (rjk2.le.eps) go to 1301
!         rjk=sqrt(rjk2)
!         costh=(rik2+rij2-rjk2)/(2.0d0*rik*rij)
!         terc=tercm(tsp(asp(i)));terd=terdm(tsp(asp(i)));terh=terhm(tsp(asp(i)))
!         g=(1.0d0+terc**2/terd**2 &
!         -terc**2/(terd**2+(terh-costh)**2))
!!         if (mode.eq.1) then
!!            zmatp=fik*g*exp(termu**term*(rij-rik)**term)
!!         else
!            zmatp=fik*g*terw
!!         endif
!         zmatm=zmatm+zmatp
! 1301    continue
! 1300 continue
!      end
